/*******************************************************************************
Copyright  1999-2003 Text Analysis International, Inc.
All rights reserved.
********************************************************************************
*
* NAME: MAIN.CPP
* FILE: main.cpp
* CR:   01/29/99 AM.
* SUBJ: Sample driver program that uses the Analyzer and KB API functions.
*       Loading an analyzer invokes the user extensions library (user/user.dll).
* ENV:  APPS - a default environment variable for VisualText applications.
*
*******************************************************************************/
#ifndef LINUX
#include <windows.h>
#endif
#include <iostream>
#include <fstream>
using namespace std;
#include <stdio.h>
#include <stdlib.h>
#include <strstream>
#include <time.h>

#include "vtapi.h"		// VISUALTEXT API INCLUDES.			// 01/31/03 AM.

#include "Uanalyze.h"

bool cmdReadargs(int,char*argv[],
		/*UP*/char*&,char*&,bool&,bool&,bool&);
void cmdHelpargs(char*);

main(
   int argc,
   char *argv[],
   char *envp[]
   )
{
// Without this, can't find the error output on the console.
#ifndef LINUX
cerr = cout;                                          // HACK. // 10/18/98 AM.
#endif

//////////////////////////////////
/// NAME OF CURRENT APP DIRECTORY
//////////////////////////////////
char *appname;
appname = "App";      // TODO: REPLACE "App" WITH THE REAL DIRECTORY NAME.

/////////////////////////////////////////////////
// READ ARGUMENTS
/////////////////////////////////////////////////
// Need to get command line arguments and/or some kind of init file.
char *input=0, *output=0;
bool develop = false;      // Development mode.
char *sequence=0;
char appdir[MAXSTR];       // Directory for the application.
appdir[0] = '\0';
bool compiled=false;       // Run compiled/interp analyzer.
bool silent=false;			// No log/debug output files.

/////////////////////////////////////////////////
// GET APP INFORMATION
/////////////////////////////////////////////////

// Get input and output filenames from command line.
if (!cmdReadargs(argc, argv,/*UP*/input,output,develop,compiled,silent))
   exit(1);

cout << "\n[Current date: " << today() << "]" << endl;

char *path = 0;
if (!(path = getenv("APPS")))
   {
   cout << "[No APPS environment variable.]" << endl;
   return 1;
   }

sprintf(appdir, "%s%c%s", path,DIR_CH,appname);

char *vtpath = 0;                                               // 08/28/02 AM.
if (!(vtpath = getenv("VISUALTEXT")))                           // 08/28/02 AM.
   {
   cout << "[No VISUALTEXT environment variable.]" << endl;
   return 1;
   }

char rfbdir[MAXSTR];		    // Directory for VisualText data.   // 08/28/02 AM.
rfbdir[0] = '\0';                                               // 08/28/02 AM.
sprintf(rfbdir, "%s%cdata%crfb%cspec",                          // 08/28/02 AM.
                          vtpath,DIR_CH,DIR_CH,DIR_CH);         // 08/28/02 AM.

// A general temporary directory.                               // 08/28/02 AM.
char *tmpdir = "c:\\temp";                                      // 08/28/02 AM.
char logfile[MAXSTR];		 // General VT log file.             // 08/28/02 AM.
logfile[0] = '\0';                                              // 08/28/02 AM.
sprintf(logfile, "%s%cvisualtext.log",tmpdir,DIR_CH);           // 08/28/02 AM.

// Set up subdirectories with default names.
// Could read a configuration file here.
// Could set up to label output files based on input file name
// or some user-supplied prefix.  Or create unique file names.
char specdir[MAXSTR];

char infile[MAXSTR];
char outfile[MAXSTR];
char seqfile[MAXSTR];

sprintf(specdir, "%s%sspec",   appdir, DIR_STR);

if (!input)
   input = "input.txt";         // Default.
if (!output)
   output = "output.txt";       // Default.

// Get analyzer-sequence file name.
if (!sequence)
   sequence = "analyzer.seq";   // Default.

sprintf(infile, "%s", input);
sprintf(outfile, "%s", output);
sprintf(seqfile, "%s%s%s", specdir,DIR_STR, sequence);

if (!path_exists(infile))
   {
   cerr << "[File/directory not found: " << infile << "]" << endl;
   return 0;
   }

/////////////////////////////////////////////////
// INITIALIZE VISUALTEXT RUNTIME SYSTEM                        // 08/27/02 AM.
/////////////////////////////////////////////////
VTRun *vtrun = new VTRun(                                      // 08/28/02 AM.
	logfile,                // Verbose/error log file.          // 08/28/02 AM.
	rfbdir,                 // VisualText dir for RFB spec.     // 08/28/02 AM.
	true                    // Build silently.                  // 08/28/02 AM.
	);

ALIST *alist = (ALIST *)vtrun->alist_;                         // 08/28/02 AM.


/////////////////////////////////////////////////
// INITIALIZE ANALYZER RUNTIME ENGINE
/////////////////////////////////////////////////
// Create and initialize an NLP object to manage text analysis.
// NOTE: This init will dynamically load the user extensions dll at
// appdir\user\debug\user.dll
NLP *nlp = new NLP(appdir, develop,silent,compiled,	// FIX.	// 01/06/03 AM.
	0,appname);																	// 01/06/03 AM.

/////////////////////////////////////////////////
// SET UP THE KNOWLEDGE BASE
/////////////////////////////////////////////////

CG cgobj(appdir,
         true,      // LOAD COMPILED KB IF POSSIBLE.
			alist);    // Give KB the handle for List Manager.    // 08/14/02 AM.
CG *cg = &cgobj;
nlp->setCG(cg);     // Give analyzer the CG REFERENCE.
if (!cg->readKB())
   {
   cerr << "[Couldn't read knowledge base.]" << endl;
   delete nlp;
   delete vtrun;                                               // 08/28/02 AM.
   return -1;
   }

// Root of the KB hierarchy.
CONCEPT *root = cg->findRoot();


/////////////////////////////////////////////////
// BUILD ANALYZER APPLICATION
/////////////////////////////////////////////////
// Create an analyzer dynamically using the sequence file and rules files
// under appdir\spec.

if (!nlp->make_analyzer(seqfile, appdir, develop,
	silent,              // Debug/log file output.              // 06/16/02 AM.
   0,
   false,               // false == Don't compile during load.
   compiled))                    // Compiled/interp analyzer.
   {
   cerr << "[Couldn't build analyzer.]" << endl;
   delete nlp;
   delete vtrun;                                               // 08/28/02 AM.
   return -1;
   }


/////////////////////////////
// TEST RULE GENERATION.
/////////////////////////////
#ifdef TEST_RUG_
CONCEPT *gram = cg->findConcept(root, "gram");
if (!gram)
   gram = cg->makeConcept(root, "gram");
RUG::rugInit(gram,cg);

// Execute rule generation.
RUG *rug = new RUG(gram, nlp, appdir, cg);
rug->rugGenerate(false,false);      // Generate ALL.
delete rug;
rug = 0;
#endif

// To have analyzer output to a stream, can rebind the cout
// stream.  // 05/05/02 AM.
char ofstr[MAXSTR];
sprintf(ofstr,"e:\\dummy.txt");
ofstream os(ofstr, ios::out);												// 08/07/02 AM.

// Testing output to buffer.
char obuf[MAXSTR];															// 05/11/02 AM.


/////////////////////////////////////////////////
// ANALYZE INPUT TEXT FILES
/////////////////////////////////////////////////

if (is_file(infile))      // Input is a single file.
   {
   // If input is from a buffer, specify its name and length.
   nlp->analyze(infile, outfile, appdir, develop,
      silent,        // Debug/log output files.                // 06/16/02 AM.
      0,            // Outdir.
      0,            // Input buffer.
      0,            // Length of input buffer, or 0.
      compiled,      // If running compiled analyzer.
		&os,				// Rebind cout output stream in analyzer  // 08/07/02 AM.
		obuf,																		// 05/11/02 AM.
		MAXSTR																	// 05/11/02 AM.
      );
   }
else      // Develop mode not on.
   {
   // Analyze file, directory, or tree of directories.
   // NOTE: This function has not been upgraded.
   // analyze_path(nlp, infile, outfile, appdir, true);
   cout << "[Not parsing directory as input]" << endl;
   }


// Can check that buffer was filled by analyzer calls such as
// cbuf() << "hello" << "\n";
//cerr << "obuf=";                                             // 05/11/02 AM.
//cerr << obuf << endl;                                        // 05/11/02 AM.


/////////////////////////////////////////////////
// CLEANUP VISUALTEXT RUNTIME
/////////////////////////////////////////////////

// This will close the user.dll for the application also.
delete nlp;         // Remove all analyzer objects.
delete vtrun;                                                  // 08/28/02 AM.
object_counts();    // Report memory leaks to standard output.

return 0;
}


/********************************************
* FN:		CMDREADARGS
* CR:		11/30/98 AM.
* SUBJ:	Get command line arguments.
* RET:	True if ok read, else false if failed.
* NOTE:	06/16/02 AM. Renamed from dosReadargs, moved to user project.
********************************************/

bool cmdReadargs(
	int argc,
	char *argv[],
	/*UP*/
	char* &input,				// Input file from args.
	char* &output,				// Output file from args.
	bool &develop,				// Development mode (output intermediate files).
	bool &compiled,	// true - compiled ana. false=interp(DEFAULT).
	bool &silent	// true == only output files specified by analyzer.
	)
{
char *ptr;
char **parg;
bool f_in  = false;
bool f_out = false;
bool flag  = false;
bool compiledck = false;	// If compiled/interp arg seen.

input = output = 0;
develop = false;				// Default is not development mode.
compiled = false;	// INTERP ANALYZER BY DEFAULT
silent = false;	// Produce debug files, etc. by default.		// 06/16/02 AM.

for (--argc, parg = &(argv[1]); argc > 0; --argc, ++parg)
	{
	// For each command line argument.
	//*gout << "command arg=" << *parg << endl;
	ptr = *parg;
	if (*ptr == '/' || *ptr == '-')	// DOS or UNIX style arg.
		{
		if (flag)
			{
			cerr << "[Error in command line args for " << argv[0]
						  << "]" << endl;
			return false;
			}
		++ptr;
		if (!strcmp_i(ptr, "in"))
			f_in = flag = true;					// Expecting input file.
		else if (!strcmp_i(ptr, "out"))
			f_out = flag = true;					// Expecting output file.
		else if (!strcmp_i(ptr, "dev"))		// 12/25/98 AM.
			{
			if (silent)
				{
				cerr << "[Ignoring /dev flag.]" << endl;
				develop = false;
				}
			else
				develop = true;					// Development mode.
			}
		else if (!strcmp_i(ptr, "silent"))	                     // 06/16/02 AM.
			{
			if (develop)
				{
				cerr << "[Ignoring /dev flag.]" << endl;
				develop = false;
				}
			silent = true;                                        // 06/16/02 AM.
			}
		else if (!strcmp_i(ptr, "interp"))	// Run interpreted analyzer.
			{
			if (compiledck)
				cerr << "[Ignoring extra /compiled or /interp flag.]" << endl;
			else
				{
				compiledck = true;
				compiled = false;
				}
			}
		else if (!strcmp_i(ptr, "compiled"))	// Run compiled analyzer.
			{
			if (compiledck)
				cerr << "[Ignoring extra /compiled or /interp flag.]" << endl;
			else
				{
				compiledck = true;
				compiled = true;
				}
			}
		}
	else if (flag)							// Expected an argument value.
		{
		if (f_in)
			{
			if (input)
				{
				cerr << "[" << argv[0] << ": Input file specified twice.]"
						  << endl;
				cmdHelpargs(argv[0]);
				return false;
				}
			// Grab value as input file.
			input = ptr;
			f_in = flag = false;
			}
		else if (f_out)
			{
			if (output)
				{
				cerr << "[" << argv[0]
						  << ": Output file specified twice.]" << endl;
				cmdHelpargs(argv[0]);
				return false;
				}
			// Grab value as output file.
			output = ptr;
			f_out = flag = false;
			}
		}
	else										// Got a "floating" value.
		{
		if (input && output)
			{
			cerr << "[" << argv[0] << ": Extra arguments.]" << endl;
			cmdHelpargs(argv[0]);
			return false;
			}
		else if (input)
			output = ptr;
		else
			input = ptr;
		}
	}
return true;
}


/********************************************
* FN:		CMDHELPARGS
* CR:		11/30/98 AM.
* SUBJ:	Print command line argument help.
* RET:	True if ok read, else false if failed.
* NOTE:	06/16/02 AM. Renamed from dosHelpargs, moved to user project.
********************************************/

void cmdHelpargs(char *name)
{
cout << name
<< " [/INTERP][/COMPILED][/IN infile] [/OUT outfile] [/DEV][/SILENT] [infile [outfile]]"
	  << endl
	  << "Note: /INTERP, the interpreted analyzer, is default."
	  << endl;
}
