###############################################
# FILE: clausefuns.pat
# SUBJ: Clause-handling functions.
# AUTH: AM
# CREATED: 29/May/02 11:34:14
# MODIFIED:
###############################################

@DECL

########
# FUNC:   NVN
# SUBJ:   Noun-verb-noun sentential pattern handling.
# CR:     05/28/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	  'reader' refers to the person being commanded or
#		spoken or written to.  Things a bit complex in here:
#	(verb adj prep)
#	(verb prep)
#	verb (adv=prep, eg "out")
#   verb (pp=prep+np)
#	Not handling verb np prep np (throw ball at me) for now.
########
nvn(L("a1"),L("np1"),L("a2"),L("vg"),L("a3"),L("np2"),L("a4"))
{
if (!L("vg"))
  return;

# Storing all this stuff in the clause now.	# 08/06/02 AM.
if (L("a1"))
  S("a1") = L("a1");
if (L("np1"))
  S("np1") = L("np1");
if (L("a2"))
  S("a2") = L("a2");
if (L("vg"))
  S("vg") = L("vg");
if (L("a3"))
  S("a3") = L("a3");
if (L("np2"))
  S("np2") = L("np2");
if (L("a4"))
  S("a4") = L("a4");

if (pnvar(L("vg"),"passive"))
  {
  if (L("np2"))
    {
	"error.txt" << "Passive followed by NP." << "\n";
	return;
	}
  "dump.txt" << "passive voice clause: ";
  if (L("np1"))
  	"dump.txt" << "np=" << pnvar(L("np1"),"$text")
	<< " ";
  "dump.txt" << "vg=" << pnvar(L("vg"),"$text")
	<< "\n";
  # HANDLING PASSIVE.  # 06/23/02 AM.
  nvnpassive(L("a1"),L("np1"),L("a2"),L("vg"),L("a3"),L("np2"),L("a4"));
  return;
  }

# Looking at ambiguous active-passive voice.	# 07/25/02 AM.
# Ex: "at the meeting held yesterday"
# Todo: more principled fetch of verb and vconj.
if (!L("np1") && !L("np2") && pnvar(L("vg"),"-edn"))
  {
  # Mark passive voice.
  pnreplaceval(L("vg"),"passive",1);
  
  # Implicit which/that clause.
  S("pattern") = "ellipted-that-clause";

  nvnpassive(L("a1"),L("np1"),L("a2"),L("vg"),L("a3"),L("np2"),L("a4"));
  return;
  }

# Active voice.

if (L("np1"))
  {
  S("actor") = L("np1");
  subjpro(L("np1"));	# 06/03/02 AM.
  }
if (L("vg"))
  S("act") = L("vg");
if (L("np2"))
  S("dobj") = L("np2");

# Semantic lookup. (Hardwiring for the moment,
# due to fast deadline.)
# go step => ptrans unit
S("particle stem") = pnvar(L("vg"),"particle stem");
S("adj stem") = pnvar(L("vg"), "adj stem");
S("act val") = pnvar(L("vg"),"act val");
S("act stem") = pnvar(L("vg"),"stem");
"dump.txt" << "nvn: act stem=" << S("act stem") << "\n";
if (!S("act val"))
  {
  if (S("act stem"))
	{
	if (S("adj stem") && S("particle stem"))
	  S("act val") = S("act stem")
		+ " " + S("adj stem")
		+ " " + S("particle stem");
	else if (S("particle stem"))
	  S("act val") = S("act stem") + " " + S("particle stem");
	}
  }

if (!pnvar(L("vg"),"act val"))	# 07/10/02 AM.
  pnreplaceval(L("vg"),"act val",S("act val"));

if (L("np2"))
  {
  S("pattern") = "nvn";	# Note the clausal pattern.

  # Domain specific, quick semantics here.
  # nvndom(L("np1"),L("vg"),L("np2"));

  S("obj stem") = pnvar(L("np2"),"stem");
  objnp(L("np2"));	# Handle pronoun object-case.
  if (!S("particle stem"))
	{
	vgadvlmulti(S("act stem"),L("a3"));	# Process adverbial.
	vgnp(S("act stem"),S("obj stem"));
	
	# Look for indirect object etc.	# 07/09/02 AM.
	nvnadvls(L("np2"),L("a4"),0);	# 07/09/02 AM.
	}
#  "dump.txt" << S("act stem")
#  	<< " " << S("obj stem")
#  	<< "\n";
  return;
  }

# No NP2, beyond this point.
S("pattern") = "nv";	# 09/27/02 AM.

if (!L("a3"))
  {
  if (S("act val"))
    return;
  if (S("act stem"))
    S("act val") = S("act stem");
  else
    S("act val") = pnvar(L("vg"),"$text");

  if (!pnvar(L("vg"),"act val"))	# 07/10/02 AM.
    pnreplaceval(L("vg"),"act val",S("act val"));
  return;	# Done.
  }

if (!pnvar(L("a3"),"pp"))	# Not a prepositional phrase.
  {
  if (!S("particle stem"))
    vgadvlmulti(S("act stem"),L("a3"));	# Process adverbial.
  # Todo: Need a clean adverbial processing fn.
  if (S("act val"))
    return;
  if (S("act stem"))
    S("act val") = S("act stem");
  else
    S("act val") = pnvar(L("vg"),"$text");

  if (!pnvar(L("vg"),"act val"))	# 07/10/02 AM.
    pnreplaceval(L("vg"),"act val",S("act val"));
  return;
  }

if (S("particle stem"))
  {
  if (S("act val"))
    return;
  if (S("act stem"))
    S("act val") = S("act stem");
  else
    S("act val") = pnvar(L("vg"),"$text");

  if (!pnvar(L("vg"),"act val"))	# 07/10/02 AM.
    pnreplaceval(L("vg"),"act val",S("act val"));
  # Todo: Need a clean adverbial processing fn.
  return;
  }

# Check for pps that are not likely verb particle+np.
if (pnvar(L("a3"),"sem") == "date")
  return;

# Look for prep dobj.

# For the moment, assume only one pp.
L("prep") = pndown(L("a3"));
if (L("prep"))
  L("np") = pnnext(L("prep"));
L("prep stem") = pnvar(L("prep"),"stem");
L("prep text") = pnvar(L("prep"),"$text");
"dump.txt" << "nvn: pp=" << L("prep text");
"dump.txt" << " " << pnvar(L("np"),"$text") << "\n";

if (!S("act val"))
  {
  if (S("act stem"))
    S("act val") = S("act stem") + " " + L("prep stem");
  else
    S("act val") = pnvar(L("vg"),"$text") + " " + L("prep stem");
  }
else
   "dump.txt" << "nvn: act val=" << S("act val") << "\n";

# Update act val in the vg.	# 07/08/02 AM.
pnreplaceval(L("vg"),"act val",S("act val"));
 
# If pronoun, do semantics on it.
objnp(L("np"));	# Handle pronoun object-case.

# If the a3 adverbial is eventive, check out any adverbials to its
# right.	# 07/08/02 AM.
L("a3") = pnnext(L("a3"));
if (!L("a3"))
  return;
nadvls(L("np"),L("a3"),L("np2"));
}

########
# FUNC:   NVNPASSIVE
# SUBJ:   Noun-verb-noun passive sentential pattern handling.
# CR:     06/23/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	  'reader' refers to the person being commanded or
#		spoken or written to.  Things a bit complex in here:
#	(verb adj prep)
#	(verb prep)
#	verb (adv=prep, eg "out")
#   verb (pp=prep+np)
#	Not handling verb np prep np (throw ball at me) for now.
########
nvnpassive(L("a1"),L("np1"),L("a2"),L("vg"),L("a3"),L("np2"),L("a4"))
{
G("oldout") << "in passive" << "\n";

S("passive") = 1;	# Flag clause as passive.

if (!L("vg"))
  return;
if (L("np2"))
  return;	# Not handling things like "john was kicked susan"
G("oldout") << "in passive1" << "\n";

# Examine the np and immediately following adverbials.
# E.g., look for eventive np, clausal adverbials etc.
nadvls(L("np1"),L("a2"),L("vg"));

L("objs") = getnodesems(L("np1"));
if (!(L("acts") = getnodesems(L("vg")) ))
  return;

S("act") = L("vg");
S("dobj") = L("np1");

if (!L("a3"))	# E.g., "john was kicked".
  {
  if (!S("act val"))
    S("act val") = actval(L("vg"),0);
  return;		# Todo: Handle case with missing actor.
  }
G("oldout") << "in passive2" << "\n";

# Look for "by np" adverbials.
L("prep") = pnvar(L("a3"),"prep stem");
if (!L("prep"))
  return;
G("oldout") << "in passive3" << "\n";
if (L("prep") != "by")
  return;
G("oldout") << "in passive4" << "\n";

# Uniformly get the actors, acts, objects.
G("oldout") << "in passive6" << "\n";
if (!(L("actors") = getnodesems(L("a3")) ))
  return;
G("oldout") << "in passive7" << "\n";

# Assign components of clause.
S("actor") = pnvar(L("a3"),"pn nps");

# Now should have: obj act by actor.
G("oldout") << "actors=" << L("actors") << "\n";
G("oldout") << "acts=" << L("acts") << "\n";
G("oldout") << "objs=" << L("objs") << "\n";
G("oldout") << "in passive last" << "\n";
}

########
# FUNC:   NADVLS
# SUBJ:   Check noun phrase and subsequent adverbials.
# CR:     06/23/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	Don't register in semantics till all clauses are parsed.
#		So this is a preliminary look at relationships.  E.g.,
#		"event in np"
#		"event of np"
# METH: Algorithmic traversal rather than pattern-based method here.
########
nadvls(L("np"),L("a"),L("end"))
{
if (!L("np"))
  return;

# Characterize np.  If eventive, note it.
if (pnvar(L("np"),"eventive"))
  L("eventive") = 1;

if (L("a"))	# Adverbials present.
  {
  "dump.txt" << "nadvls: np=" << pnvar(L("np"),"$text") << "\n";
  "dump.txt" << "nadvls: a=" << pnvar(L("a"),"$text") << "\n";
  # Look for actors/objects, if np was eventive.
  if (L("eventive")
   && pnvar(L("a"),"pp") )
    {
	"dump.txt" << "eventive+pp" << "\n";
#	L("prep") = pnvar(L("a"),"prep stem");
	L("prep") = ppprep(L("a"));
	"dump.txt" << "nadvls: prep=" << L("prep") << "\n";
	if (L("prep") == "in"
	 || L("prep") == "of"
	 || L("prep") == "by"	# 07/09/02 AM.
	 )
	   {
	   "dump.txt" << "eventive+in/of np" << "\n";
	   # We have "event in/of np".
	   # Note that the np has an actor role in event.
	   L("pn nps") = pnvar(L("a"),"pn nps");
	   if (L("pn nps"))
	     {
		 "dump.txt" << "event+prep+np" << "\n";
		 # Cross-link eventive node and actor node.
		 pnreplaceval(L("np"),"pn actors", L("pn nps"));	# Cross link.
		 pnreplaceval(L("pn nps"),"pn eventive",L("np"));	# Cross link.
		 }
	   }
	}
  
  # Look for clausal adverbials.
  L("a2") = pnnext(L("a"));
  if (L("a2") == L("end"))
    L("a2") = 0;

  # Link to sub clause also...
  # np that x y z.
  }

if (!L("a2") || !L("pn nps"))
  return;

# Pattern: np that clause
if (pnvar(L("a2"),"pattern") != "that-clause")
  return;
if (!(L("clause") = pnvar(L("a2"),"clause") ))
  return;
"dump.txt" << "found np+that+clause" << "\n";
# Cross-link np with its that-clause.
pnreplaceval(L("pn nps"),"that-clause",L("clause"));
pnreplaceval(L("clause"),"np-that",L("pn nps"));
}

########
# FUNC:   NVNADVLS
# SUBJ:   Check noun phrase and subsequent adverbials.
# CR:     07/09/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	Don't register in semantics till all clauses are parsed.
#		So this is a preliminary look at relationships.  E.g.,
# METH: Algorithmic traversal rather than pattern-based method here.
########
nvnadvls(L("np"),L("a"),L("end"))
{
if (!L("np"))
  return;
if (!L("a"))
  return;

# Characterize np.  If eventive, note it.
if (pnvar(L("np"),"eventive"))
  L("eventive") = 1;

"dump.txt" << "nvnadvls:" << "\n";

# Traverse subtree of adverbials.
L("n") = L("a");
L("dn") = 0;
while (L("n"))
  {
#  "dump.txt" << "nvna n=" << pnvar(L("n"),"$text") << "\n";
  # Process bona fide adverbial.
  # Traverse to next adverbial.
  L("nm") = pnname(L("n"));
  if (L("nm") == "_adv")
    {
	# Process adverb.
	return;
	}
  else if (L("nm") != "_advl")
    return;
  # We're in adverbial.
  L("dn") = pndown(L("n"));
  L("nm") = pnname(L("dn"));
  if (L("nm") == "_advl")
    nvnadvls(L("np"),L("dn"),L("end"));
  else
    {
	# Process proper adverbial here...
	"dump.txt" << "proper advl=" << pnvar(L("n"),"$text") << "\n";
	if (pnvar(L("n"),"pp"))	# In prep phrase.
	  {
	  L("prep") = ppprep(L("n"));
	  L("np") = pnvar(L("n"),"pn nps");
#	  if (L("prep") == "in"	# Not restricting. # 07/24/02 AM.
#	   || L("prep") == "of")
	     {
	     if (L("np"))
		   S("iobj") = L("np");
		 }
	  }
	}
  L("n") = pnnext(L("n"));	# Traverse current phrase of advls.
  if (!L("n"))
    return;
  if (L("n") == L("end"))
    return;
  }
}



########
# FUNC:   NVJ
# SUBJ:   Noun-verb-adj sentential pattern handling.
# CR:     05/31/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	  'reader' refers to the person being commanded or spoken to.
########
nvj(L("a1"),L("np1"),L("a2"),L("vg"),L("a3"),L("adj1"),L("a4"))
{
if (!L("adj1"))
  return;	# Adj is the key to this pattern.

if (L("vg"))
  {
  if (pnvar(L("vg"),"passive"))
    return; # Not handling yet.
  }

S("restriction") = 1;  # Flag restriction or condition on np.
  S("pattern") = "nvj";	# Note the clausal pattern.

# Active voice.
  {
  if (L("np1"))
    S("actor") = L("np1");
  if (L("vg"))
    S("act") = L("vg");

  # Semantic lookup. (Hardwired for now.)
  if (L("vg"))
    S("act stem") = pnvar(L("vg"),"stem");
  else
    S("act stem") = "be";

  # Look for advs, as in "book closest to shelf".
  if (L("a4"))	# Assume just one for the momment.
  	{
	if (!pnvar(L("a4"),"pp"))	# Not a prepositional phrase.
	  S("adv stem") = pnvar(L("a4"),"stem");
	else	# Look for dobj in pps.
	  {
	  # For the moment, assume only one pp.
	  L("prep") = pndown(L("a4"));
	  if (L("prep"))
	    L("np") = pnnext(L("prep"));
	  L("prep stem") = pnvar(L("prep"),"stem");
	  L("prep text") = pnvar(L("prep"),"$text");
	  "dump.txt" << "pp=" << L("prep text");
	  "dump.txt" << " " << pnvar(L("np"),"$text") << "\n";
	  

	  # If pronoun, do semantics on it.
	  if (pnvar(L("np"),"pro"))
	    {
	    objnp(L("np"));	# Handle pronoun object-case.
		
		S("act val") = S("act stem")
		  + " " + pnvar(L("adj1"),"$text")
		  + " " + L("prep text");
		}
	  } # end look for dobj in pps.
	} # end a4
  }  # end active voice.


# Domain-specific, quick semantics here.
#nvjdom(L("np1"),L("vg"),L("adj1"));

}  # end function.


########
# FUNC:   OBJNP
# SUBJ:   Handle object-case np.
# CR:     06/01/02 AM.
# INPUT:  
# OUTPUT: S("obj val") = Slot fill for object case.
#		  S("dobj") = Point to direct object np node.
# NOTE:	  'reader' refers to the person being commanded or spoken to.
#		06/03/02 AM. Renamed from OBJPRO.
########

objnp(L("np"))
{
if (!L("np"))
  return;

# Not a pronoun.  Handle it.
if (!pnvar(L("np"),"pro"))
  {
#  "dump.txt" << "objnp=" << pnvar(L("np"),"$text") << "\n";
  L("np stem") = pnvar(L("np"),"np stem");
  L("con") = 0;
  if (L("np stem"))
    {
	L("np stem") = strtolower(L("np stem"));
	L("con") = findconceptwithparent(L("np stem"),"advsem","compass");
	if (!L("con"))
	  L("con") = findconceptwithparent(L("np stem"),"nounsem","location");
	}
  if (L("con")
   || pnvar(L("np"),"sem") == "location")
   	  {
	  # Found compass point, location, or coordinate.
#	  "dump.txt" << "found location" << "\n";
	  S("loc slot") = "location";
	  S("loc val") = pnvar(L("np"),"$text");
	  return;
	  }
  S("dobj") = L("np");
  return;
  }

S("dobj") = L("np");

# Assume it's corroborated as object-case pronoun.
L("pro stem") = pnvar(L("np"),"stem");
if (L("pro stem") == "me"
  || L("pro stem") == "myself" )
  S("obj val") = "writer";
else if (L("pro stem") == "you"
  || L("pro stem") == "yourself" )
  S("obj val") = "reader";
else
  S("obj val") = L("pro stem");
}



########
# FUNC:   SUBJPRO
# SUBJ:   Handle subject-case pronoun.
# CR:     06/03/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	  'reader' refers to the person being commanded or spoken to.
########

subjpro(L("np"))
{
if (!L("np"))
  return;
if (!pnvar(L("np"),"pro"))
  return;

# Assume it's corroborated as object-case pronoun.
L("pro stem") = pnvar(L("np"),"stem");
if (L("pro stem") == "I"
  || L("pro stem") == "myself" )
  S("actor val") = "writer";
else if (L("pro stem") == "you"
  || L("pro stem") == "yourself" )
  S("actor val") = "reader";
else
  S("actor val") = L("pro stem");
}

########
# FUNC:   VERBPREPIDIOM
# SUBJ:   Identify verb-prep idioms.
# CR:     06/01/02 AM.
# INPUT:  
# OUTPUT: 1 if an idiom, else 0.
# NOTE:   May want to place these in kb and look them up.
########

verbprepidiom(L("verb"),L("prep"))
{
if (!L("verb") || !L("prep"))
  return 0;
if (L("verb") == "wake" && L("prep") == "up")
  return 1;
return 0;
}

########
# FUNC:   VGADVL
# SUBJ:   Process a verb+adverbial.
# CR:     06/01/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:   Not looking for particle-np extensions to a verb.
#		  This function only handles one adverbial.
########
vgadvl(L("act stem"), L("nadvl"))
{
if (!L("nadvl"))
  return;

if (pnvar(L("nadvl"),"pp"))
  {
  # Todo: Process prep phrase.
  return;
  }

# Handle mainly preps, adjs, advs...


# Adverbial is a particle.  Eg, "out".
S("adv stem") = pnvar(L("nadvl"),"stem");
if (!S("adv stem"))
  return;

# Hard-wired look for some verb+particle idioms, for now...
L("idiom") = verbprepidiom(S("act stem"),S("adv stem"));
if (L("idiom"))
  {
  if (S("particle stem"))
    return;
  S("particle stem") = S("adv stem");
  S("adv stem") = 0;
  if (S("act val"))
    S("act val") = S("act val") + " " + S("particle stem");
  else
	S("act val") = L("act stem") + " " + S("particle stem");
  return;
  }

# Todo: Look these up in the kb.
# Look at some general semantics of particles.
if (S("adv stem"))
  {
#  "dump.txt" << "adv stem=" << S("adv stem") << "\n";
  L("sem con") = findconceptwithparent(
  					S("adv stem"),"advsem","compass");
  if (!L("sem con"))
    L("sem con") = findconceptwithparent(
					S("adv stem"),"advsem","rel");
  if (L("sem con"))	# Found concept with 'compass' as parent.
    {
	S("adv stem") = strtolower(conceptname(L("sem con")));
    S("adv sem") = "direction";
    return;
	}
  }
}

########
# FUNC:   VGADVLMULTI
# SUBJ:   Process a verb+adverbials.
# CR:     06/02/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:   Not looking for particle-np extensions to a verb.
#		For multi, only looking at glomming directions, like
#		"north east".
########
vgadvlmulti(L("act stem"), L("nadvl"))
{
if (!L("nadvl"))
  return;

if (pnvar(L("nadvl"),"pp"))
  {
  # Todo: Process prep phrase.
  return;
  }

# Handle mainly preps, adjs, advs...

# If single adverbial, go handle it.
L("stems") = pnvar(L("nadvl"),"stems");
if (!L("stems"))
  {
  vgadvl(L("act stem"), L("nadvl"));
  return;
  }

"dump.txt" << "[vgadvlmulti: " << pnvar(L("nadvl"),"$text") << "]\n";

L("sems") = pnvar(L("nadvl"),"sems");

L("str") = 0;	# Build direction value here.
L("astr") = 0;	# Alternate hypothesis as list.
L("ii") = 0;	# Counter for advls.
L("tot") = arraylength(L("stems"));
L("done") = 0;
# Find the first compass guy, if any.
while (!L("done"))
  {
  L("stem") = L("stems")[L("ii")];
  L("sem") = L("sems")[L("ii")];
  L("sem con") = findconceptwithparent(
  					L("stem"),"advsem","compass");
  if (L("sem con"))	# Found concept with 'compass' as parent.
    {
	L("str") = L("astr") = conceptname(L("sem con"));
	L("done") = 1;
	}
  else
    getadvlsem(L("stem"),L("sem"));

  if (++L("ii") >= L("tot"))
    L("done") = 1;
  }

 if (L("ii") >= L("tot"))
  {
  if (L("str"))
  	{
	# One compass direction total.
	S("adv stem") = L("str");
    S("adv sem") = "direction";
	}
  return;
  }

# Find and catenate subsequent compass points.
L("done") = 0;
while (!L("done"))
  {
  L("stem") = L("stems")[L("ii")];
  L("sem")  = L("sems")[L("ii")];
  L("sem con") = findconceptwithparent(
  					L("stem"),"advsem","compass");
  if (L("sem con"))
    {
	# Catenate compass points.  eg, "E" + "NE".
	L("str") = L("str") + conceptname(L("sem con"));
	L("astr") = L("astr") + ", " + conceptname(L("sem con"));
	"dump.txt" << "cat=" << L("str") << "\n";

    if (++L("ii") >= L("tot"))
      L("done") = 1;
	}
  else
    L("done") = 1; # At non compass point advl.

  }

if (L("str"))
  {
  L("str") = strtolower(L("str"));
  # Multi compass directon.
  # Check for valid compass point!
  L("sem con") = findconceptwithparent(L("str"),"advsem","compass");
  if (!L("sem con"))
    {
	# Error. Bad glommed compass point.
	# Todo: NLP++ needs a good way to send error to
    # VisualText log window.
	"error.txt" << "[vgadvlmulti: Bad compass point="
        << L("str") << "]\n";
	S("adv stem") = strtolower(L("astr"));	# Use comma list.
	S("adv sem") = "direction";
    }
  else
    {
	S("adv stem") = L("str");
    S("adv sem") = "direction";
	}
  }

# Process any remaining adverbials.
while (L("ii") < L("tot"))
  {
  L("stem") = L("stems")[L("ii")];
  L("sem") = L("sems")[L("ii")];
  getadvlsem(L("stem"),L("sem"));
  ++L("ii");
  }
}

########
# FUNC:   GETADVLSEM
# SUBJ:   Process an adverbial.
# CR:     06/03/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:   Compass points handled elsewhere.
########
getadvlsem(L("stem"),L("sem"))
{
if (!L("stem"))
  return;

#"dump.txt" << "[getadvlsem: stem=" << L("stem")
#	<< ",sem=" << L("sem") << "\n";

if (!L("sem"))	# Try to figure out semantics.
  {
  L("sem con") = findconceptwithparent(
					L("stem"),"advsem","rel");
  if (L("sem con"))
    {
	L("dir val") = strtolower(conceptname(L("sem con")));
    if (S("direction val"))
	  S("direction val") = S("direction val") + " and " + L("dir val");
	else
	  S("direction val") = L("dir val");
#	S("adv stem") = strtolower(conceptname(L("sem con")));
#   S("adv sem") = "direction";
	}
  }
else if (L("sem") == "speed")
  {
  S("speed slot") = "speed";
  S("speed val") = L("stem");
  }
}


########
# FUNC:   VGNP
# SUBJ:   Process a verb+noun.
# CR:     06/01/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:  
########

vgnp(L("act stem"),L("obj stem"))
{
# Todo: Look up in the kb.
  if (L("act stem") == "go"
   || L("act stem") == "move"
   || L("act stem") == "turn"
   || L("act stem") == "jump"
   )
    {
	if (L("obj stem") == "step")
	  {
	  # move distance.
	  S("act sem") = "move";
	  S("obj sem") = "distance";
      S("obj slot") = "distance";
	  }
	}
}

########
# FUNC:   GETNODESEMS
# SUBJ:   Get semantic list for given node.
# CR:     06/23/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:  
########

getnodesems(L("node"))
{
if (!L("node"))
  return 0;
if (L("sems") = pnvar(L("node"),"sems"))
  return L("sems");
if (L("sems") = pnvar(L("node"),"stems"))
  return L("sems");
if (L("sems") = pnvar(L("node"),"sem"))
  return L("sems");
if (L("sems") = pnvar(L("node"),"stem"))
  return L("sems");
return 0;
}

########
# FUNC:   PPPREP
# SUBJ:   Get prep info from adverbial.
# CR:     07/09/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:  
########

ppprep(L("a"))
{
if (!L("a"))
  return 0;
L("prep stem") = pnvar(L("a"),"prep stem");
if (L("prep stem"))
  return L("prep stem");
L("n") = pndown(L("a"));
if (!L("n"))
  return 0;
if (!pnvar(L("n"),"prep"))
  return 0;
L("prep stem") = pnvar(L("n"), "stem");
if (!L("prep stem"))
  L("prep stem") = pnvar(L("n"),"$text");
return L("prep stem");
}

########
# FUNC:   ACTVAL
# SUBJ:   Get and set text for an act.
# CR:     07/11/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:  
########

actval(L("n"),L("prep"))
{
if (!L("n"))
  return 0;
L("act val") = pnvar(L("n"),"act val");
if (L("act val"))
  return L("act val");

L("stem") = pnvar(L("n"),"stem");  # Verb stem, presumably.
if (L("prep"))
  {
  if (L("stem"))
    L("act val") = L("stem") + " " + L("prep");
  }
else if (L("stem"))
  L("act val") = L("stem");
else
  L("act val") = pnvar(L("n"),"$text");

# I don't like putting it here, as it can be a bigger item than
# covered by the node, which is typically just a verb group.
if (L("act val"))
  pnreplaceval(L("n"),"act val", L("act val"));

return L("act val");
}

@@DECL

@CODE
G("hello") = 0;
@@CODE
