###############################################
# FILE: domfuns.pat
# SUBJ: Domain-specific semantic and discourse processing.
# AUTH: AM
# CREATED: 28/May/02 13:33:38
# NOTE:	Demoing with "ball world".
###############################################

@DECL
########
# FUNC:   DOMIFTHEN
# SUBJ:   Domain-specific handling for if-then statement.
# CR:     05/28/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	  'reader' refers to the person being commanded or spoken to.
########

domifthen(
  L("ifclause"),
  L("thenclause")
  )
{
if (!L("ifclause") || !L("thenclause"))
  return;

G("oldout") << "\nCONDITION" << "\n";
G("oldout") << "---------";
domcomplex(L("ifclause"));

G("oldout") << "\nACTION" << "\n";
G("oldout") << "------";
domcomplex(L("thenclause"));
}

########
# FUNC:   DOMCONDITION
# SUBJ:   Domain-specific handling for if or when statement.
# CR:     06/03/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	  'reader' refers to the person being commanded or spoken to.
########

domcondition(
  L("condclause")
  )
{
if (!L("condclause"))
  return;

G("oldout") << "\nCONDITION" << "\n";
G("oldout") << "---------";
domcomplex(L("condclause"));
}


########
# FUNC:   DOMCLAUSE
# SUBJ:   
# CR:     05/28/02 AM.
# INPUT:  
# OUTPUT: 
########

domclause(L("nclause"))
{
if (!L("nclause"))
  return;

if (pnvar(L("nclause"),"adj role"))	# Linking clause.
  {
  domlinkclause(L("nclause"));
  return;
  }

L("n act") = pnvar(L("nclause"),"act");
L("act stem") = pnvar(L("nclause"),"act stem");
if (!L("n act") && !L("act stem"))
  return;

G("oldout") << "\nevent:" << "\n";

G("oldout") << "\t" << "actor: \t";
L("n actor") = pnvar(L("nclause"),"actor");
if (!L("n actor"))
  G("oldout") << "reader" << "\n";
else
   {
   L("actor val") = pnvar(L("nclause"),"actor val");
   if (!L("actor val"))
     L("actor val") = pnvar(L("n actor"),"$text");
   G("oldout") << strtolower(L("actor val"));
   if (pnvar(L("n actor"),"wh start"))
     G("oldout") << "?";
   G("oldout") << "\n";
   }

L("act sem") = pnvar(L("nclause"),"act sem");
L("act val") = pnvar(L("nclause"),"act val");
L("act text") = pnvar(L("n act"),"$text");
if (!L("act val"))
  {
  if (L("act sem"))
    L("act val") = L("act sem");
  else if (L("act stem"))
    {
    L("act val") = L("act stem");
	}
  else
    L("act val") = L("act text");
  }
G("oldout") << "\t" << "act: \t"
    << strtolower(L("act val")) << "\n";

L("n dobj") = pnvar(L("nclause"),"dobj");
if (L("n dobj"))
  {
  L("obj text") = pnvar(L("n dobj"),"$text");
  L("obj slot") = pnvar(L("nclause"),"obj slot");
  L("obj val")  = pnvar(L("nclause"),"obj val");
  if (!L("obj slot"))
    L("obj slot") = "obj";
  if (!L("obj val"))
    L("obj val") = L("obj text");
  if (L("obj val") == "it")	# PRONOUN RESOLUTION.
    {
	# Hacking this, will generalize later.
	# Look up clause in kb.
	# Move backward, looking for objects.
	L("kbclause") = pnvar(L("nclause"),"kb concept");
	L("ret") = resolveintra(L("kbclause"),"obj","text");
	if (L("ret"))
	  L("obj val") = L("ret");
	}
  G("oldout") << "\t" << strtolower(L("obj slot")) << ": \t"
		<< strtolower(L("obj val"))
		<< "\n";
  }

L("loc slot") = pnvar(L("nclause"),"loc slot");
L("loc val") = pnvar(L("nclause"),"loc val");
if (L("loc val"))
  {
  G("oldout") << "\t" << strtolower(L("loc slot")) << ": \t"
		<< strtolower(L("loc val"))
		<< "\n";
  }

L("speed slot") = pnvar(L("nclause"),"speed slot");
L("speed val") = pnvar(L("nclause"),"speed val");
if (L("speed val"))
  {
  G("oldout") << "\t" << strtolower(L("speed slot")) << ": \t"
		<< strtolower(L("speed val"))
		<< "\n";
  }

L("direction val") = pnvar(L("nclause"),"direction val");
if (L("direction val"))
  {
  G("oldout") << "\t" << "direction: \t"
		<< strtolower(L("direction val"))
		<< "\n";
  }

L("adv sem") = pnvar(L("nclause"),"adv sem");
L("adv stem") = pnvar(L("nclause"),"adv stem");
if (L("adv sem") && L("adv stem"))
  {
  G("oldout") << "\t" << strtolower(L("adv sem")) << ": \t"
		<< strtolower(L("adv stem"))
		<< "\n";
  }
}



########
# FUNC:   DOMLINKCLAUSE
# SUBJ:   Clause with a linking verb (copula).
# CR:     05/31/02 AM.
# INPUT:  
# OUTPUT: 
########

domlinkclause(L("nclause"))
{
if (!L("nclause"))
  return;
L("n adj") = pnvar(L("nclause"),"adj role");
if (!L("n adj"))
  return;

L("n actor") = pnvar(L("nclause"),"actor");
if (!L("n actor"))
  L("n actor") = pnvar(L("nclause"),"np resolver");
if (!L("n actor"))
  return;  # Unresolved actor.

L("actor val") = pnvar(L("nclause"),"actor val");
L("act val")   = pnvar(L("nclause"),"act val");
L("obj val")   = pnvar(L("nclause"),"obj val");
L("obj slot")  = pnvar(L("nclause"),"obj slot");

if (!L("actor val"))
  {
  L("actor text") = pnvar(L("n actor"),"$text");
  if (L("actor text"))
    L("actor val") = L("actor text");
  if (pnvar(L("n actor"),"wh start"))
    L("actor q") = 1;
  }

if (!L("act val"))
  {
  L("n act")    = pnvar(L("nclause"),"act");
  L("act stem") = pnvar(L("nclause"),"act stem");
  L("act sem")  = pnvar(L("nclause"),"act sem");
  if (L("n act"))
    L("act text") = pnvar(L("n act"),"$text");
  if (L("act sem"))
    L("act val") = L("act sem");
  else if (L("act stem"))
    {
    L("act val") = L("act stem");
	}
  else if (L("act text"))
    L("act val") = L("act text");
  else
    L("act val") = "be";
  L("act val") = L("act val") + " " + pnvar(L("n adj"),"$text");
  }

if (!L("obj val"))
  {
  L("n dobj") = pnvar(L("nclause"),"dobj");
  if (L("n dobj"))
    {
    L("obj text") = pnvar(L("n dobj"),"$text");
    if (L("obj text"))
      L("obj val") = L("obj text");
	}
  }
if (!L("obj slot"))
  L("obj slot") = "obj";

G("oldout") << "\nstate:" << "\n";

G("oldout") << "\t" << "actor: \t"
	<< strtolower(L("actor val"));
if (L("actor q"))
  G("oldout") << "?";
G("oldout") << "\n";

G("oldout") << "\t" << "act: \t"
    << strtolower(L("act val")) << "\n";

if (L("obj slot") && L("obj val"))
  G("oldout") << "\t" << strtolower(L("obj slot")) << ": \t"
		   << strtolower(L("obj val"))
		   << "\n";

L("loc slot") = pnvar(L("nclause"),"loc slot");
L("loc val") = pnvar(L("nclause"),"loc val");
if (L("loc val"))
  {
  G("oldout") << "\t" << strtolower(L("loc slot")) << ": \t"
		<< strtolower(L("loc val"))
		<< "\n";
  }

L("adv sem") = pnvar(L("nclause"),"adv sem");
L("adv stem") = pnvar(L("nclause"),"adv stem");
if (L("adv sem") && L("adv stem"))
  {
  G("oldout") << "\t" << strtolower(L("adv sem")) << ": \t"
		<< strtolower(L("adv stem"))
		<< "\n";
  }
}


########
# FUNC:   DOMCOMPLEX
# SUBJ:   
# CR:     05/28/02 AM.
# INPUT:  
# OUTPUT: 
########

domcomplex(L("ncomplex"))
{
if (!L("ncomplex"))
  return;

if (pnname(L("ncomplex")) == "_clause")
  {
  domclause(L("ncomplex"));
  return;
  }

L("n") = pndown(L("ncomplex"));
while (L("n"))
  {
  L("name") = pnname(L("n"));
  if (L("name") == "_clause")
    domclause(L("n"));
  else if (L("name") == "_conj")
    G("oldout") << "\nconj: "
		<< strtolower(pnvar(L("n"),"$text")) << "\n";
  L("n") = pnnext(L("n"));
  }
}

########
# FUNC:   DOMQUESTION
# SUBJ:   
# CR:     05/29/02 AM.
# INPUT:  
# OUTPUT: 
########

domquestion(L("nclause"))
{
G("oldout") << "\nQUESTION" << "\n";
G("oldout") << "---------";
if (!L("nclause"))
  return;
domcomplex(L("nclause"));
}

########
# FUNC:   DOMIMPERATIVE
# SUBJ:   
# CR:     05/29/02 AM.
# INPUT:  
# OUTPUT: 
########

domimperative(L("nclause"))
{
G("oldout") << "\nCOMMAND" << "\n";
G("oldout") << "-------";
if (!L("nclause"))
  return;
domcomplex(L("nclause"));
}

########
# FUNC:   DOMDECLARATIVE
# SUBJ:   
# CR:     05/29/02 AM.
# INPUT:  
# OUTPUT: 
########

domdeclarative(L("nclause"))
{
if (!L("nclause"))
  return;
#if (pnvar(L("nclause"),"restriction"))
#  G("oldout") << "\nCONDITION" << "\n"
#  		   <<   "---------";
#else
#  G("oldout") << "\nFACT" << "\n"
#		   << "----";
domcomplex(L("nclause"));
}

########
# FUNC:   RESOLVEINTRA
# SUBJ:   Resolve an item within same sentence.
# CR:     05/31/02 AM.
########

resolveintra(
  L("kbclause"),  # The current kb clause.
  L("typ"), # The type of thing to find. eg, "obj".
  L("attr") # Attribute to return, eg., "text".
  )
{
if (!L("kbclause") || !L("typ"))
  return 0;

L("ret") = 0;
L("kbclause") = prev(L("kbclause"));
while (L("kbclause"))
  {
  if (L("kbitem") = down(L("kbclause")))
    {
    # Find last item in clause.
	L("next") = next(L("kbitem"));
	while (L("next"))
	  {
	  L("kbitem") = L("next");
	  L("next") = next(L("kbitem"));
	  }
	# Now at last item, go backward.
	L("done") = 0;
	while (!L("done"))
	  {
	  if (strcontains(L("typ"),conceptname(L("kbitem"))))
	    {
		# RESOLVE TO FIRST ITEM FOUND, GOING BACKWARD.
	    L("ret") = strval(L("kbitem"),L("attr"));
		L("done") = 1;
	    }
	  else
	    {
	    L("kbitem") = prev(L("kbitem"));
	    if (!L("kbitem"))
	      L("done") = 1;
	    }
	  }
    }
  L("kbclause") = prev(L("kbclause"));
  }
return L("ret");
}

########
# FUNC:   DOMOBJECT
# SUBJ:   Domain-specific handling of word.
# CR:     08/18/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	Some quick-and-dirty...
########
domobject(L("text"),L("node"),L("nounsem"))
{
if (!L("text") || !L("node"))
  return;

if (L("nounsem"))
  {
  L("name") = conceptname(L("nounsem"));
  pnreplaceval(L("node"),"dom object",L("name"));
  }

# Look for object in domain-specific hierarchy.
# Todo: Define this global for your domain objects.
if (!G("domain hierarchy"))
  return;

if (findconcept(G("domain hierarchy"),L("text")) )
  {
  pnreplaceval(L("node"),"dom slot",L("text"));
  return;
  }
L("stem") = stem(L("text"));
"stem.txt" << L("stem") << "\n";
if (findconcept(G("domain hierarchy"),L("stem")) )
  pnreplaceval(L("node"),"dom slot",L("stem"));
}


@@DECL