###############################################
# FILE: funs.pat
# SUBJ: A repository for user-defined functions.
# AUTH: AM
# CREATED: 27/Feb/02 10:50:18
# MODIFIED:
###############################################

@DECL
########
# FUNC:   SETPASSIVE
# INPUT:  G("be")
#         G("verb") -- verb's node.
# OUTPUT: G("passive") = 1 or 0.
# WARN:   Uses G("tmp") as a local var.
########
setpassive()
{
G("passive") = 0;
if (G("be") && G("verb"))
  {
  if (pnvar(G("verb"),"-edn")
   || pnvar(G("verb"),"-en"))
    G("passive") = 1;
  else if (!pnvar(G("verb"),"inf")) # Not an exception.
    {
	G("tmp") = strtolower(pnvar(G("verb"),"$text"));
	if (strendswith(G("tmp"),"ed")
	 || pnvar(G("verb"),"-edn") )
	  G("passive") = 1;
	}
  }
else if (G("verb"))
  {
  # Can still look for -en feature.
  if (pnvar(G("verb"),"-en"))
    G("passive") = 1;
  }
}

########
# FUNC:   VGAGREE
# CR:	  05/03/02 AM.
# ARGS:	  Top-level nodes assumed to bear info.
# RET:	  L("ret")[0] - 1 if agreement, else 0.
#		  L("ret")[1] - 1 if passive voice, else 0.
#		  L("ret")[2] - "past", "present", "future", 0.
#		  L("ret")[3] = "inf","-en","-ed","-edn","-ing","-s", or 0. 
#				First element's conjugation. (see notes).
#		  May want progressive, etc. "was eating..."
#	Would like the FAILURE POINT, so that I know where to flag
#	no glomming verbs left and no glomming verbs right.
# NOTE:	For constructs like "Should John be eating", will
#	consider that "be eating" has good agreement.  But
#	may want to return a flag saying it is "incomplete".
#	Should John be eating => inf-start.
#	Has John eaten => en-start.
#	Was John eating => ing-start.
#	Was John eaten => en-start.
# Some hints:
#	modal => needs inf.
#	have  => needs en.
#	be    => needs en (passive) or ing.
#   being => needs en.
# OPT: May want to internalize this in NLP++ for speed, at some
#	point.
########

vgagree(L("m"),L("h"),L("b"),L("being"),L("v"))
{
#if (L("m"))
#  "vg.txt" << "m=" << pnvar(L("m"),"$text") << "\n";
#if (L("v"))
#  "vg.txt" << "v=" << pnvar(L("v"),"$text") << "\n";

L("ret")[0] = 1; # Agreement till proved otherwise.
L("ret")[3] = 0; # Set return vals to zero.

L("need") = 0;	# What's needed next.
	# Vals = "inf","en","en-ing".
L("first") = 0; # First element seen.
L("last") = 0;  # Last element seen.

if (L("m")) # MODAL
  {
  # Need to set can-could type of modality here, or "future".

    L("need") = "inf";
  if (!L("first"))
    L("first") = L("m");
  L("last") = L("m");
  }

if (L("h"))	# HAVE
  {
  if (L("need") == "inf"
   && !pnvar(L("h"),"inf") )
    {
	# eg, "WILL HAD".
	L("ret")[0] = 0;	# NO AGREEMENT.
	return L("ret");
	}

  L("need") = "en";
  if (!L("first"))
    L("first") = L("h");
  L("last") = L("h");
  }

if (L("b"))	# BE
  {
  if (L("need") == "inf")
    {
    if (!pnvar(L("b"),"inf"))
      {
	  # eg, "WILL BEEN".
	  L("ret")[0] = 0;	# NO AGREEMENT.
	  return L("ret");
	  }
	}
  else if (L("need") == "en"  # BUG FIX.	# 07/10/02 AM.
   && !pnvar(L("b"),"-en") )
    {
	# eg, "HAVE ARE".
	L("ret")[0] = 0;	# NO AGREEMENT.
	return L("ret");
	}
  
  L("need") = "en-ing";
  if (!L("first"))
    L("first") = L("b");
  L("last") = L("b");
  }

if (L("being"))	# BEING
  {
  if (L("need") == "inf")
    {
	# eg, "WILL BEING".
	L("ret")[0] = 0;	# NO AGREEMENT.
	return L("ret");
	}
  else if (L("need") == "en")	# BUG FIX.	# 07/10/02 AM.
    {
	# eg, "HAVE BEING".
	L("ret")[0] = 0;	# NO AGREEMENT.
	return L("ret");
	}
  # else 0 or "en-ing" are OK.

  L("need") = "en";
  if (!L("first"))
    L("first") = L("being");
  L("last") = L("being");
  }

if (L("v"))	# VERB
  {
  L("vsuff") = vconj(L("v"));
  if (!L("vsuff"))
      {
	  # eg, "WAS BEEN".
	  L("ret")[0] = 0;	# NO AGREEMENT.
	  return L("ret");
	  }

  if (L("need") == "inf")
    {
    if (L("vsuff") != "inf")
      {
	  # eg, "WILL ATE".
	  L("ret")[0] = 0;	# NO AGREEMENT.
	  return L("ret");
	  }
	}
  else if (L("need") == "en")	# BUG FIX.	# 07/10/02 AM.
    {
    if (L("vsuff") != "-edn"
	 && L("vsuff") != "-en" )
      {
	  # eg, "HAVE ATE".
	  L("ret")[0] = 0;	# NO AGREEMENT.
	  return L("ret");
	  }
	}
  else if (L("need") == "en-ing")
    {
    if (L("vsuff") == "-edn"
	 || L("vsuff") == "-en" )
	  {
	  L("ret")[1] = 1;  # SETS PASSIVE VOICE FOR VG.
	  }
	else if (L("vsuff") == "-ing")
	  {
	  }
	else
      {
	  # eg, "WAS ATE".
	  L("ret")[0] = 0;	# NO AGREEMENT.
	  return L("ret");
	  }
	}
  # else 0 is OK.
  
  if (!L("first"))
    L("first") = L("v");
  L("last") = L("v");
  }

# TODO: Use first and last to determine need to the left
# and need to the right of this verb group.  eg, for
# resolving question syntax to left, unknown words to right.

return L("ret");
}

########
# FUNC:   VEN
# ARGS:	  Verb node.
# RET:	  1 if verb+en or +edn
# ASS:	Assumes all irregular verb forms are in (l_)common.pat
# (Could place this knowledge in kb also, for easy reference...)
# May not work as desired for verbs like have, be.
########

ven(L("v"))
{
if (!L("v"))
  return 0;
if (pnvar(L("v"),"sem") == "be") # Special case.
  return 0;
if (pnvar(L("v"),"-edn")	# Irregular verb +edn ending.
 || pnvar(L("v"),"-en") )	# Irregular +en
  return 1;
if (pnvar(L("v"),"inf")		# Exception like "need".
 || pnvar(L("v"),"-ed") )	# Irregular like "ate".
  return 0;
 
# Need a convention for getting just the verb's text.
# eg in something like "haven't".
L("vtext") = nodetext(L("v"));
if (!L("vtext"))
  return 0;
if (strendswith(L("vtext"),"ed"))
  return 1;	# "Normal" verb with ambiguous ed ending.
return 0;
}

########
# FUNC:	VCONJ
# SUBJ:	Get verb conjugation.
# ARGS:	Verb node.
# RET:	"inf","-s","-ing","-edn","-ed","-en", or 0.
# ASS:	Assumes all irregular verb forms are in (l_)common.pat
#		(Could place this knowledge in kb also, for easy
#		reference...)
#		May not work as desired for verbs like be.
########

vconj(L("v"))
{
if (!L("v"))
  return 0;
if (pnvar(L("v"),"sem") == "be") # Special case.
  return 0;

# OPT: May want to collapse all this to one var!
if (pnvar(L("v"),"-edn"))	# Irregular verb +edn ending.
  return "-edn";
if (pnvar(L("v"),"-en"))	# Irregular +en
  return "-en";
if (pnvar(L("v"),"inf"))	# Exception like "need".
  return "inf";
if (pnvar(L("v"),"-ed"))	# Irregular like "ate".
  return "-ed";
if (pnvar(L("v"),"-ing"))
  return "-ing";
if (pnvar(L("v"),"-s"))
  return "-s";

# NOT IRREGULAR, SO USE LITERAL TEXT.
# Need a convention for getting just the verb's text.
# eg in something like "haven't".
L("vtext") = nodetext(L("v"));
#"vg.txt" << "vconj text=" << L("vtext") << "\n";
if (!L("vtext"))
  return 0;
if (strendswith(L("vtext"),"ed"))
  return "-edn";	# "Normal" verb with ambiguous ed ending.
if (strendswith(L("vtext"),"ing"))
  return "-ing";
if (suffix(L("vtext"),"s"))	# "s" is a proper suffix.
  return "-s";
return "inf";	# Assume found uninflected form.
}

########
# FUNC: NUMBER
# SUBJ:	Check singular/plural/any for noun.
# ARGS: Noun node.
# RET:	"singular", "plural", "any", or 0.
# ASS:	Should return any for mass nouns, things like sheep...
# TODO:	Irregular plurals.  radii, cherubim, etc.
########

number(L("n"))
{
if (!L("n"))
  return 0;
if (pnvar(L("n"),"-s"))
  return "plural";
L("num") = pnvar(L("n"),"number");
if (L("num"))
  return L("num");

L("ntext") = nodetext(L("n"));
if (!L("ntext"))
  return 0;
L("stem") = pnvar(L("n"),"stem");
if (!L("stem"))
  L("stem") = stem(L("ntext"));
if (strendswith(L("ntext"),"s")
 && L("stem") != L("ntext") )
  return "plural";
return "singular";
}


########
# FUNC:	SINGULAR
# SUBJ:	True if noun can be singular.
# ARGS: Noun node.
# RET:	1 or 0.
# ASS:	Should return true for mass nouns, things like sheep...
# TODO:	Irregular plurals.  radii, cherubim, etc.
########

singular(L("n"))
{
if (!L("n"))
  return 0;
if (pnvar(L("n"),"-s"))
  return 0;

L("ntext") = nodetext(L("n"));
if (!L("ntext"))
  return 0;
L("stem") = pnvar(L("n"),"stem");
if (!L("stem"))
  L("stem") = stem(L("ntext"));
if (strendswith(L("ntext"),"s")
 && L("stem") != L("ntext") )
  return 0;
return 1;
}


########
# FUNC:	PLURAL
# SUBJ:	True if noun can be plural.
# ARGS: Noun node.
# RET:	1 or 0.
# ASS:	Should return true for mass nouns, things like sheep...
# TODO:	Irregular plurals.  radii, cherubim, etc.
########

plural(L("n"))
{
if (!L("n"))
  return 0;
if (pnvar(L("n"),"-s"))
  return 1;

L("ntext") = nodetext(L("n"));
if (!L("ntext"))
  return 0;
L("stem") = pnvar(L("n"),"stem");
if (!L("stem"))
  L("stem") = stem(L("ntext"));
if (strendswith(L("ntext"),"s")
 && L("stem") != L("ntext") )
  return 1;
return 0;
}

########
# FUNC:   NODETEXT
# ARGS:	  Get text of a node.
# RET:	  text or 0.
# ASS:	Using variable called "text" if node's text was
#	created explicitly.
# WARN:	GETTING LOWERCASED TEXT.
########

nodetext(L("n"))
{
if (!L("n"))
  return 0;
L("text") = pnvar(L("n"),"text"); # If var present, use it.
if (!L("text"))
  return strtolower(pnvar(L("n"),"$text")); # Get from tokens.
return L("text");
}

########
# FUNC:   NODESTEXT
# ARGS:	  Get text of a range of nodes, inclusive.
# RET:	  text or 0.
# WARN:	GETTING LOWERCASED TEXT.
# OPT:	Should get start and end offsets, then copy part of
#		text buffer (if raw text sought).
########

nodestext(L("n1"), L("n2"))
{
if (!L("n1"))
  return 0;
L("end") = L("n2");
if (L("end"))
  L("end") = pnnext(L("end"));	# One past end.
#L("text") = 0;	# Collect text here.
while (L("n1")	# For safety.
  && L("n1") != L("end"))
  {
#  "dump.txt" << nodetext(L("n1")) << "\n";
  if (L("text"))
    L("text") = L("text") + " " + nodetext(L("n1"));
  else
    L("text") = nodetext(L("n1"));
  L("n1") = pnnext(L("n1"));
  }
return L("text");
}

########
# FUNC:   NODESEM
# SUBJ:	  Quick and dirty fetch of "semantic" word for a node.
# CR:	  08/05/02 AM.
# RET:	  text or 0.
# WARN:	GETTING LOWERCASED TEXT.
########

nodesem(L("n"))
{
if (!L("n"))
  return 0;
if (L("sem") = pnvar(L("n"),"sem"))
  return L("sem");
if (L("sem") = pnvar(L("n"),"stem"))
  return L("sem");
if (L("text") = pnvar(L("n"),"text")) # Get from var.
  return L("text");
return strtolower(pnvar(L("n"),"$text")); # Get from tokens.
}

########
# FUNC:   LOOKUPALPHA
# SUBJ:   Look up an alphabetic word.
# CR:     03/20/02 AM.
# INPUT:  Assumes there's a word-node to attach results to.
# OUTPUT: 
# WARN:   Modifies vars in the given node.
########

lookupalpha(
	L("text"),	# Lowercased text for word.
	L("node")	# The node representing the word.
	)
{
if (!(G("wordcon") = dictfindword(L("text"))))
	return;	# Not handling unknown words here.
pnreplaceval(L("node"),"wordcon", G("wordcon"));

# Grab some attributes from kb.	# 06/24/02 AM.
if (L("nsem") = conval(G("wordcon"),"nounsem"))	# 06/24/02 AM.
  {
  pnreplaceval(L("node"),"nounsem",L("nsem"));	# 06/24/02 AM.
  if (inhierarchy(L("nsem"),"event"))	# 06/26/02 AM.
      pnreplaceval(L("node"),"eventive",1);	# 06/24/02 AM.
  }
domobject(L("text"),L("node"),L("nsem"));

G("vals") = findvals(G("wordcon"), "pos");
G("pos num") = 0;
G("pos") = 0;
while (G("vals"))
	{
	G("val") = getstrval(G("vals"));
	if (G("val") == "noun")
		{
		pnreplaceval(L("node"),"noun", 1);
		G("pos") = "_noun";
		eventivenoun(L("text"),L("node"));
		}
	else if (G("val") == "verb")
		{
		pnreplaceval(L("node"),"verb", 1);
		G("pos") = "_verb";
		
		# Any -ing verb can be an eventive noun.
		if (strendswith(L("text"),"ing"))
			pnreplaceval(L("node"),"eventive",1);
		}
	else if (G("val") == "adjective")
		{
		pnreplaceval(L("node"),"adj", 1);
		G("pos") = "_adj";
		}
	else if (G("val") == "adverb")
		{
		pnreplaceval(L("node"),"adv", 1);
		G("pos") = "_adv";
		}
	else if (G("val") == "prep")
		{
		pnreplaceval(L("node"),"prep", 1);
		G("pos") = "_prep";
		}
	else if (G("val") == "pronoun")
		{
		pnreplaceval(L("node"),"pro", 1);
		G("pos") = "_pro";
		}
	else if (G("val") == "conj"
	 || G("val") == "conjunction")	# 03/20/02 AM.
		{
		pnreplaceval(L("node"),"conj", 1);
		G("pos") = "_conj";
		}
	else if (G("val") == "interj")
		{
		pnreplaceval(L("node"),"interj", 1);
		G("pos") = "_interj";
		}
	else if (G("val") == "det")
		{
		pnreplaceval(L("node"),"det", 1);
		G("pos") = "_det";
		}
	else
	    {
		# Hmm.  Found "abbreviation" as one unhandled
		# pos.  # 12/08/01 AM.
#		"misc.txt" << "abbreviation=" << L("text") << "\n";
		pnreplaceval(L("node"),"abbr",1);
		# G("pos") = 0;
		}
	G("vals") = nextval(G("vals"));
	++G("pos num");
	}
pnreplaceval(L("node"),"pos num", G("pos num"));

if (G("pos num") > 0 && !pnvar(L("node"),"stem"))
  {
  if (G("pos"))
    pnreplaceval(L("node"),"pos",G("pos"));
  if (G("stem") = stem(L("text")))
  	pnreplaceval(L("node"),"stem",G("stem"));
  else
    pnreplaceval(L("node"),"stem",L("text"));
  }
}

########
# FUNC:   MONTHTONUM
# SUBJ:   Convert month to number.
# CR:     04/19/02 AM.
# INPUT:  
# OUTPUT: 
########

monthtonum(
	L("text")	# Lowercased text for word.
	)
{
if (L("text") == "january" || L("text") == "jan")
  return 1;
else if (L("text") == "february" || L("text") == "feb")
  return 2;
else if (L("text") == "march" || L("text") == "mar")
  return 3;
else if (L("text") == "april" || L("text") == "apr")
  return 4;
else if (L("text") == "may")
  return 5;
else if (L("text") == "june" || L("text") == "jun")
  return 6;
else if (L("text") == "july" || L("text") == "jul")
  return 7;
else if (L("text") == "august" || L("text") == "aug")
  return 8;
else if (L("text") == "september" || L("text") == "sep"
	|| L("text") == "sept")
  return 9;
else if (L("text") == "october" || L("text") == "oct")
  return 10;
else if (L("text") == "november" || L("text") == "nov")
  return 11;
else if (L("text") == "december" || L("text") == "dec")
  return 12;
else
  return 0;
}

########
# FUNC:   NPSEG
# SUBJ:   See if seg is a possible np.
# CR:     05/14/02 AM.
# INPUT:  
# OUTPUT: 
########

npseg(
	L("node")
	)
{
if (!L("node"))
  return 0;
if (pnname(L("node")) != "_seg")
  return 0;
if (pnvar(L("node"),"seg type") == "np")
  return 1;
if (pnvar(L("node"),"unknown"))
  return 1;
if (pnvar(L("node"),"noun"))
  return 1;
return 0;
}

########
# FUNC:   VGSEG
# SUBJ:   See if seg is a possible vg.
# CR:     05/29/02 AM.
# INPUT:  
# OUTPUT: 
########

vgseg(
	L("node")
	)
{
if (!L("node"))
  return 0;
if (pnname(L("node")) != "_seg")
  return 0;
if (pnvar(L("node"),"seg type") == "vg")
  return 1;
if (pnvar(L("node"),"seg len") != 1)
  return 0;
if (pnvar(L("node"),"verb"))
  return 1;
if (pnvar(L("node"),"seg verbs"))
  return 1;
return 0;
}

########
# FUNC:   INHIERARCHY
# SUBJ:   See if given concept is under a concept by name.
# CR:     06/26/02 AM.
# INPUT:  
# OUTPUT: 
########

inhierarchy(L("con"),L("parent_str"))
{
if (!L("parent_str"))
  return 0; # false.
while (L("con"))
  {
  if (conceptname(L("con")) == L("parent_str"))
    return 1; # true.
  L("con") = up(L("con"));
  }
return 0; # false.
}

########
# FUNC:   UNKNOWNWORD
# SUBJ:   Process word not in dictionary.
# CR:     07/10/02 AM.
# INPUT:  
# OUTPUT: 
########

unknownword(L("text"),	# The word
	L("n")) # The node
{
if (!L("text") || !L("n"))
  return;
# Some morphology...
if (eventivenoun(L("text"),L("n")))
  {
  pnreplaceval(L("n"),"noun",1);
  pnreplaceval(L("n"),"eventive",1);
  }
}

########
# FUNC:   EVENTIVENOUN
# SUBJ:   Morphology for eventive nouns.
# CR:     07/10/02 AM.
# INPUT:  
# OUTPUT: 
########

eventivenoun(L("text"),	# The word
	L("n")) # The node
{
if (!L("text") || !L("n"))
  return;
# Some morphology...
#if (strendswith(L("text"),"ity")
# || strendswith(L("text"),"ion")
# || strendswith(L("text"),"ities")
# || strendswith(L("text"),"ance")
# || strendswith(L("text"),"phobia"))
#  pnreplaceval(L("n"),"eventive",1);
}

########
# FUNC:   MERGENPS
# SUBJ:   Merge nps and lists from two nodes.
# CR:     07/11/02 AM.
# INPUT:  
# OUTPUT: 
########

mergenps(L("n1"), L("n2"))
{
if (!L("n1") && !L("n2"))
  return 0;
if (!L("n1"))
  return pnvar(L("n2"),"nps");
if (!L("n2"))
  return pnvar(L("n1"),"nps");

# Both present.
L("nps1") = pnvar(L("n1"),"nps");
L("nps2") = pnvar(L("n2"),"nps");
L("count1") = arraylength(L("nps1"));
if (L("count1") <= 0)
  L("count1") = 1;
L("count2") = arraylength(L("nps2"));
if (L("count2") <= 0)
  L("count2") = 1;
if (!L("nps1"))
  {
  L("nps1") = L("n1");
  L("count1") = 1;
  }
if (!L("nps2"))
  {
  L("nps2") = L("n2");
  L("count2") = 1;
  }

S("count") = L("count1") + L("count2"); # RETURN VALUE.
# Build in reverse order, assume lists are reverse order.
# (Merge two arrays....should have its own fn.)
L("ii") = -1;
L("jj") = L("count2") - 1;
while (++L("ii") < L("count1"))
  L("nps2")[++L("jj")] = L("nps1")[L("ii")];
return L("nps2");
}

########
# FUNC:   COLLECTNPS
# SUBJ:   Gather NPS from a range of nodes.
# CR:     07/11/02 AM.
# INPUT:  
# OUTPUT: 
# NOTE:	Collects in REVERSE order, because np lists collected
#	in reverse by the grammar, currently.
########

collectnps(L("n1"), L("n2"))
{
# n1 to n2 inclusive.
# Assume n1 can be null, as the "end" of the range.
if (!L("n2"))
  return;
if (L("n1"))
  L("end") = pnprev(L("n1"));
# else L("end") = 0;

L("nps") = 0;
L("count") = 0;
L("n") = L("n2");
while (L("n") != L("end"))
  {
  L("name") = pnname(L("n"));
  if (L("tmp") = pnvar(L("n"),"nps"))	# Has a list of nps already.
    {
	# Merge it with the list being built.
	}
  else if (L("name") == "_np"
   || L("name") == "_nps")
    {
	# Some rule hasn't bothered building an np list.
	
	}
  L("n") = pnprev(L("n"));
  }
}

@@DECL

@CODE
G("hello") = 0;
@@CODE
