###############################################
# FILE: xmlrecursive.pat
# SUBJ: Recursive traversal to output embedded tags.
# AUTH: AM
# CREATED: 16/Dec/02 20:30:09
# MODIFIED:
###############################################

@DECL
#############
# FN:   XMLRECURSION
# SUBJ: Traverse parse tree to output embedded tags.
# CR:   12/16/02 AM.
# NOTE: To minimize recursion, each first child will manage
#		its list of siblings.
#		Using the global G("oxml") stream for printout.
#############

xmlrecursion(
	L("n"),	# Current node.
	L("root")	# 1 if a "root", else 0.
	)
{
# Consider current node.
if (!L("n"))
  return;

# Check if printable.
L("name") = pnname(L("n"));
L("tag") = 0;
if (G("NP"))	# Analyzer caller says to print out NP nodes.
  {
# && (L("name") == "_nps" || L("name") == "_np") )
#  L("tag") = "np";
  if (L("name") == "_phone")
    L("tag") = "phone";
  else if (L("name") == "_url")
    L("tag") = "url";
  }
else if (G("NOUN")
 && (L("name") == "_noun") )
  L("tag") = "noun";

if (L("tag"))	# If printable.
  {
  printfiller(L("n"),G("oend"),G("oxml"));
  G("oxml") << "<" << L("tag") << ">";
  
  # Update right edge so filler won't be printed again.
  G("oend") = pnvar(L("n"),"$ostart");
  }

# If a leaf, print it.
if (!(L("childs") = pndown(L("n")) ))
  {
  if (!L("tag"))
    printfiller(L("n"),G("oend"),G("oxml"));
  G("oxml") << pnvar(L("n"),"$text");
  G("oend") = pnvar(L("n"),"$oend");
  }
else	# Else, traverse its subtree.
  xmlrecursion(L("childs"),0);

# If printable, print its end tag.
if (L("tag"))
  G("oxml") << "</" << L("tag") << ">";

# If first child and not a root, traverse siblings.
if (!pnprev(L("n")) && !L("root"))
  {
  while (L("n") = pnnext(L("n")) )
    xmlrecursion(L("n"),1);
  }
}

#############
## FN:   PRINTFILLER
## SUBJ: Print filler between prior and current node.
## CR:   12/15/02 AM.
#############
printfiller(
	L("node"),	# Current node to print.
	L("oend"),	# Right edge offset of last printed text.
	L("out")	# Output stream.
	)
{
	L("ostart") = pnvar(L("node"),"$ostart");
	if (L("oend") == L("ostart"))
	  return;
    if (L("oend")
	 && (L("ostart") - L("oend") - 1) )
	  {
	  # Get snippet of input text between last and current output node.
	  if (!(L("snip") = inputrange(L("oend")+1,L("ostart")-1) ))
	    "err.txt" << "snip: " << L("oend") << "," << L("ostart") << "\n";
	  if (L("n") = strchrcount(L("snip"),"\n"))	# If snippet has a newline.
	    {
		while (L("n")-- > 0)
	      L("out") << "\n";			# Preserve newline.
		}
	  else
	    # Note: Even if there wasn't a space in the snippet, we don't know what got
		# excised in there, so printing space for safety.  For example, if the parse excised
		# a paren, then no space means two alphabetics may be inappropriately glommed together.
		# (When excising, could flag if there is a need to separate what's left with a space...)
	    L("out") << " ";			# Preserve space.
	  }
}

@CODE
if (!G("xml recursive"))
  exitpass();

# Set up the recursion here.
G("oend") = 0;	# End offset of previously printed node.
G("oxml") = "embed.txt";
xmlrecursion(pnroot(), 1);
@@CODE
